@@ -2,6 +2,7 @@
 
 /*
  * Copyright (c) 2009, Sun Microsystems, Inc.
+ * Copyright (c) 2010-2018 Red Hat, Inc. and/or its affiliates.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -95,6 +96,13 @@ enum xdr_op {
  * This is the number of bytes per unit of external data.
  */
 #define BYTES_PER_XDR_UNIT (4)
+
+/*
+ * constants specific to the xdr "protocol"
+ */
+#define XDR_FALSE (0)
+#define XDR_TRUE (1)
+
 /* Taken verbatim from a system xdr.h, which carries a BSD-style
  * license (Matt) */
 /*
@@ -109,12 +117,23 @@ enum xdr_op {
 		   * BYTES_PER_XDR_UNIT)
 #endif
 
+/* XDR vector buffer types */
+typedef enum vio_type {
+	VIO_HEADER,             /* header buffer before data */
+	VIO_DATA,               /* data buffer */
+	VIO_TRAILER_LEN,	/* length field for following TRAILER buffer */
+	VIO_TRAILER,            /* trailer buffer after data */
+} vio_type;
+
 /* XDR buffer vector descriptors */
 typedef struct xdr_vio {
 	uint8_t *vio_base;
 	uint8_t *vio_head;	/* minimum vio_tail (header offset) */
-	uint8_t *vio_tail;
+	uint8_t *vio_tail;	/* end of the used part of the buffer */
 	uint8_t *vio_wrap;	/* maximum vio_tail */
+	uint32_t vio_length;	/* length of buffer, used for vector
+				   pre-allocation */
+	vio_type vio_type;	/* type of buffer */   
 } xdr_vio;
 
 /* vio_wrap >= vio_tail >= vio_head >= vio_base */
@@ -125,6 +144,7 @@ typedef struct xdr_vio {
 #define UIO_FLAG_GIFT		0x0004
 #define UIO_FLAG_MORE		0x0008
 #define UIO_FLAG_REALLOC	0x0010
+#define UIO_FLAG_REFER		0x0020
 
 struct xdr_uio;
 typedef void (*xdr_uio_release)(struct xdr_uio *, u_int);
@@ -180,6 +200,14 @@ typedef struct rpc_xdr {
 		/* new vector and refcounted interfaces */
 		bool (*x_getbufs)(struct rpc_xdr *, xdr_uio *, u_int);
 		bool (*x_putbufs)(struct rpc_xdr *, xdr_uio *, u_int);
+		/* Force a new buffer to start (or fail) */
+		bool (*x_newbuf)(struct rpc_xdr *);
+		/* Return the count of buffers in the vector from pos */
+		int (*x_iovcount)(struct rpc_xdr *, u_int, u_int);
+		/* Fill xdr_vio with buffers from pos */
+		bool (*x_fillbufs)(struct rpc_xdr *, u_int , xdr_vio *, u_int);
+		/* Allocate bufs for headers and trailers and insert into vio */
+		bool (*x_allochdrs)(struct rpc_xdr *, u_int , xdr_vio *, int);
 	} *x_ops;
 	void *x_public; /* users' data */
 	void *x_private; /* pointer to private data */
@@ -323,15 +351,22 @@ xdr_putlong(XDR *xdrs, const long *lp)
 		(*(xdrs)->x_ops->x_control)(xdrs, req, op)
 #define xdr_control(xdrs, req, op) XDR_CONTROL(xdrs, req, op)
 
-/*
- * Solaris strips the '_t' from these types -- not sure why.
- * But, let's be compatible.
- */
-#define xdr_rpcvers(xdrs, versp) xdr_u_int32(xdrs, versp)
-#define xdr_rpcprog(xdrs, progp) xdr_u_int32(xdrs, progp)
-#define xdr_rpcproc(xdrs, procp) xdr_u_int32(xdrs, procp)
-#define xdr_rpcprot(xdrs, protp) xdr_u_int32(xdrs, protp)
-#define xdr_rpcport(xdrs, portp) xdr_u_int32(xdrs, portp)
+#define XDR_NEWBUF(xdrs)	\
+	(*(xdrs)->x_ops->x_newbuf)(xdrs)
+#define xdr_newbuf(xdrs, pos) XDR_NEWBUF(xdrs)
+
+#define XDR_IOVCOUNT(xdrs, pos, len) \
+	(*(xdrs)->x_ops->x_iovcount)(xdrs, pos, len)
+#define xdr_iovcount(xdrs, pos, len) XDR_IOVCOUNT(xdrs, pos, len)
+
+#define XDR_FILLBUFS(xdrs, pos, iov, len)	\
+	(*(xdrs)->x_ops->x_fillbufs)(xdrs, pos, iov, len)
+#define xdr_fillbufs(xdrs, pos, iov, len) XDR_FILLBUFS(xdrs, pos, iov, len)
+
+#define XDR_ALLOCHDRS(xdrs, pos, iov, iov_count)	\
+	(*(xdrs)->x_ops->x_allochdrs)(xdrs, pos, iov, iov_count)
+#define xdr_allochdrs(xdrs, pos, iov, iov_count) \
+	XDR_ALLOCHDRS(xdrs, pos, iov, iov_count)
 
 /*
  * Support struct for discriminated unions.
@@ -407,14 +442,10 @@ xdr_inline_encode(XDR *xdrs, size_t count)
 #define IXDR_GET_BOOL(buf)  ((bool)IXDR_GET_LONG(buf))
 #define IXDR_GET_ENUM(buf, t)  ((t)IXDR_GET_LONG(buf))
 #define IXDR_GET_U_LONG(buf)  ((u_long)IXDR_GET_LONG(buf))
-#define IXDR_GET_SHORT(buf)  ((short)IXDR_GET_LONG(buf))
-#define IXDR_GET_U_SHORT(buf)  ((u_short)IXDR_GET_LONG(buf))
 
 #define IXDR_PUT_BOOL(buf, v)  IXDR_PUT_LONG((buf), (v))
 #define IXDR_PUT_ENUM(buf, v)  IXDR_PUT_LONG((buf), (v))
 #define IXDR_PUT_U_LONG(buf, v)  IXDR_PUT_LONG((buf), (v))
-#define IXDR_PUT_SHORT(buf, v)  IXDR_PUT_LONG((buf), (v))
-#define IXDR_PUT_U_SHORT(buf, v) IXDR_PUT_LONG((buf), (v))
 
 /*
  * In-line routines for vector encode/decode of primitive data types.
@@ -596,13 +627,18 @@ xdr_putenum(XDR *xdrs, enum_t enumv)
 static inline bool
 xdr_getbool(XDR *xdrs, bool_t *ip)
 {
-	return xdr_getuint32(xdrs, (uint32_t *)ip);
+	uint32_t lv;
+
+	if (!xdr_getuint32(xdrs, &lv))
+		return (false);
+	*ip = lv ? XDR_TRUE : XDR_FALSE;
+	return (true);
 }
 
 static inline bool
 xdr_putbool(XDR *xdrs, bool_t boolv)
 {
-	return xdr_putuint16(xdrs, (uint32_t)boolv);
+	return xdr_putuint16(xdrs, boolv ? XDR_TRUE : XDR_FALSE);
 }
 
 #define XDR_GETBOOL(xdrs, boolp) xdr_getbool(xdrs, boolp)
@@ -619,38 +655,14 @@ extern bool xdr_int(XDR *, int *);
 extern bool xdr_u_int(XDR *, u_int *);
 extern bool xdr_long(XDR *, long *);
 extern bool xdr_u_long(XDR *, u_long *);
-extern bool xdr_short(XDR *, short *);
-extern bool xdr_u_short(XDR *, u_short *);
-extern bool xdr_int16_t(XDR *, int16_t *);
-extern bool xdr_u_int16_t(XDR *, u_int16_t *);
-extern bool xdr_int32_t(XDR *, int32_t *);
-extern bool xdr_u_int32_t(XDR *, u_int32_t *);
-extern bool xdr_uint32_t(XDR *, u_int32_t *);
-extern bool xdr_int64_t(XDR *, int64_t *);
-extern bool xdr_u_int64_t(XDR *, u_int64_t *);
-extern bool xdr_uint64_t(XDR *, u_int64_t *);
-extern bool xdr_bool(XDR *, bool_t *);
-extern bool xdr_enum(XDR *, enum_t *);
-extern bool xdr_array(XDR *, char **, u_int *, u_int, u_int, xdrproc_t);
-extern bool xdr_union(XDR *, enum_t *, char *, const struct xdr_discrim *,
-		      xdrproc_t);
-extern bool xdr_char(XDR *, char *);
-extern bool xdr_u_char(XDR *, u_char *);
-extern bool xdr_vector(XDR *, char *, u_int, u_int, xdrproc_t);
 extern bool xdr_float(XDR *, float *);
 extern bool xdr_double(XDR *, double *);
-extern bool xdr_quadruple(XDR *, long double *);
-extern bool xdr_reference(XDR *, char **, u_int, xdrproc_t);
-extern bool xdr_pointer(XDR *, char **, u_int, xdrproc_t);
+extern bool xdr_reference(XDR *, void **, u_int, xdrproc_t);
+extern bool xdr_pointer(XDR *, void **, u_int, xdrproc_t);
 extern bool xdr_wrapstring(XDR *, char **);
-extern bool xdr_hyper(XDR *, quad_t *);
-extern bool xdr_u_hyper(XDR *, u_quad_t *);
 extern bool xdr_longlong_t(XDR *, quad_t *);
 extern bool xdr_u_longlong_t(XDR *, u_quad_t *);
 
-#define xdr_quad_t  xdr_int64_t
-#define xdr_uquad_t xdr_uint64_t
-
 __END_DECLS
 
 /*
