@@ -83,17 +83,36 @@ svc_rpc_gss_cmpf(const struct opr_rbtree_node *lhs,
 		 const struct opr_rbtree_node *rhs)
 {
 	struct svc_rpc_gss_data *lk, *rk;
+	gss_union_ctx_id_desc *gss_ctx_lk, *gss_ctx_rk;
+	uint64_t mech_type_lk, internal_ctx_id_lk;
+	uint64_t mech_type_rk, internal_ctx_id_rk;
+
 
 	lk = opr_containerof(lhs, struct svc_rpc_gss_data, node_k);
 	rk = opr_containerof(rhs, struct svc_rpc_gss_data, node_k);
+	gss_ctx_lk = (gss_union_ctx_id_desc *) (lk->ctx);
+	gss_ctx_rk = (gss_union_ctx_id_desc *) (rk->ctx);
+	mech_type_lk = (uint64_t)(uintptr_t)gss_ctx_lk->mech_type;
+	mech_type_rk = (uint64_t)(uintptr_t)gss_ctx_rk->mech_type;
+	internal_ctx_id_lk = (uint64_t)(uintptr_t)gss_ctx_lk->internal_ctx_id;
+	internal_ctx_id_rk = (uint64_t)(uintptr_t)gss_ctx_rk->internal_ctx_id;
 
-	if (lk->hk.k < rk->hk.k)
-		return (-1);
+	/* Move left OR right based on mech type */
+	if (mech_type_lk < mech_type_rk)
+		return -1;
 
-	if (lk->hk.k == rk->hk.k)
-		return (0);
+	if (mech_type_lk > mech_type_rk)
+		return 1;
 
-	return (1);
+	/* If mech_type are equal, move based on internal_ctx_id */
+	if (internal_ctx_id_lk < internal_ctx_id_rk)
+		return -1;
+
+	if (internal_ctx_id_lk > internal_ctx_id_rk)
+		return 1;
+
+	/* If we have reached here, both mech_type and id are equal */
+	return 0;
 }
 
 static void
@@ -103,6 +122,11 @@ authgss_hash_init()
 
 	mutex_lock(&authgss_hash_st.lock);
 
+	if (authgss_hash_st.initialized) {
+		mutex_unlock(&authgss_hash_st.lock);
+		return;
+	}
+
 	code =
 	    rbtx_init(&authgss_hash_st.xt, svc_rpc_gss_cmpf,
 		      __svc_params->gss.ctx_hash_partitions,
@@ -133,13 +157,6 @@ authgss_hash_init()
 	mutex_unlock(&authgss_hash_st.lock);
 }
 
-#define cond_init_authgss_hash() { \
-		do { \
-			if (!authgss_hash_st.initialized) \
-				authgss_hash_init(); \
-		} while (0); \
-	}
-
 struct svc_rpc_gss_data *
 authgss_ctx_hash_get(struct rpc_gss_cred *gc)
 {
@@ -149,10 +166,11 @@ authgss_ctx_hash_get(struct rpc_gss_cred *gc)
 	struct authgss_x_part *axp;
 	struct rbtree_x_part *t;
 
-	cond_init_authgss_hash();
+	authgss_hash_init();
 
 	gss_ctx = (gss_union_ctx_id_desc *) (gc->gc_ctx.value);
 	gk.hk.k = gss_ctx_hash(gss_ctx);
+	gk.ctx = (gc->gc_ctx.value);
 
 	t = rbtx_partition_of_scalar(&authgss_hash_st.xt, gk.hk.k);
 	mutex_lock(&t->mtx);
@@ -181,7 +199,7 @@ authgss_ctx_hash_set(struct svc_rpc_gss_data *gd)
 	gss_union_ctx_id_desc *gss_ctx;
 	bool rslt;
 
-	cond_init_authgss_hash();
+	authgss_hash_init();
 
 	gss_ctx = (gss_union_ctx_id_desc *) (gd->ctx);
 	gd->hk.k = gss_ctx_hash(gss_ctx);
@@ -210,7 +228,7 @@ authgss_ctx_hash_del(struct svc_rpc_gss_data *gd)
 	struct rbtree_x_part *t;
 	struct authgss_x_part *axp;
 
-	cond_init_authgss_hash();
+	authgss_hash_init();
 
 	t = rbtx_partition_of_scalar(&authgss_hash_st.xt, gd->hk.k);
 	mutex_lock(&t->mtx);
@@ -265,7 +283,7 @@ void authgss_ctx_gc_idle(void)
 	struct svc_rpc_gss_data *gd;
 	int ix, cnt, part;
 
-	cond_init_authgss_hash();
+	authgss_hash_init();
 
 	for (ix = 0, cnt = 0, part = IDLE_NEXT();
 	     ((ix < authgss_hash_st.xt.npart) &&
