@@ -101,6 +101,7 @@ xdr_rpc_gss_buf(XDR *xdrs, gss_buffer_t buf, u_int maxsize)
 	case XDR_DECODE:
 		return (xdr_rpc_gss_decode(xdrs, buf));
 	case XDR_FREE:
+		mem_free(buf->value, buf->length);
 		return (TRUE);
 	};
 	return (FALSE);
@@ -267,7 +268,7 @@ xdr_rpc_gss_wrap(XDR *xdrs, xdrproc_t xdr_func, void *xdr_ptr,
 	 * If it's privacy, and NEWBUF is supported (because xdrs is a vector)
 	 * then NEWBUF will have allocated the new buffer.
 	 */
-	vector = (svc == RPCSEC_GSS_SVC_INTEGRITY) || XDR_NEWBUF(xdrs);
+	vector = (svc == RPCSEC_GSS_SVC_INTEGRITY);
 
 	/* Marshal rpc_gss_data_t (sequence number + arguments).
 	 * If it's a vector, the response has been marshalled into a new
@@ -305,7 +306,7 @@ xdr_rpc_gss_wrap(XDR *xdrs, xdrproc_t xdr_func, void *xdr_ptr,
 			xv_count = data_count + 2;
 			gv_count = data_count + 1;
 			after_data = data_count;
-		} else if (svc == RPCSEC_GSS_SVC_PRIVACY) {
+		} else /* svc == RPCSEC_GSS_SVC_PRIVACY */ {
 			/* Add header, padding, and trailer for the wrap */
 			xv_count = data_count + 3;
 			gv_count = data_count + 3;
